@@ -176,10 +176,7 @@ impl Source {
 mod tests {
     use super::*;
     use byteorder::{ByteOrder, LittleEndian, WriteBytesExt};
-    use ckb_db::RocksDB;
-    use ckb_db_schema::COLUMNS;
     use ckb_hash::blake2b_256;
-    use ckb_store::{data_loader_wrapper::DataLoaderWrapper, ChainDB};
     use ckb_traits::{CellDataProvider, HeaderProvider};
     use ckb_types::{
         bytes::Bytes,
@@ -200,8 +197,29 @@ mod tests {
     use proptest::{collection::size_range, prelude::*};
     use std::collections::HashMap;
 
-    fn new_store() -> ChainDB {
-        ChainDB::new(RocksDB::open_tmp(COLUMNS), Default::default())
+    #[derive(Default, PartialEq, Eq, Clone)]
+    struct MockDataLoader {
+        headers: HashMap<Byte32, HeaderView>,
+    }
+
+    impl CellDataProvider for MockDataLoader {
+        fn get_cell_data(&self, _out_point: &OutPoint) -> Option<Bytes> {
+            None
+        }
+
+        fn get_cell_data_hash(&self, _out_point: &OutPoint) -> Option<Byte32> {
+            None
+        }
+    }
+
+    impl HeaderProvider for MockDataLoader {
+        fn get_header(&self, block_hash: &Byte32) -> Option<HeaderView> {
+            self.headers.get(block_hash).cloned()
+        }
+    }
+
+    fn new_mock_data_loader() -> MockDataLoader {
+        MockDataLoader::default()
     }
 
     fn build_cell_meta(capacity_bytes: usize, data: Bytes) -> CellMeta {
@@ -244,7 +262,9 @@ mod tests {
         let resolved_cell_deps = vec![];
         let group_inputs = vec![];
         let group_outputs = vec![];
+        let data_loader = new_mock_data_loader();
         let mut load_cell = LoadCell::new(
+            &data_loader,
             &outputs,
             &resolved_inputs,
             &resolved_cell_deps,
@@ -285,7 +305,9 @@ mod tests {
         let resolved_cell_deps = vec![];
         let group_inputs = vec![];
         let group_outputs = vec![];
+        let data_loader = new_mock_data_loader();
         let mut load_cell = LoadCell::new(
+            &data_loader,
             &outputs,
             &resolved_inputs,
             &resolved_cell_deps,
@@ -374,7 +396,9 @@ mod tests {
         let resolved_cell_deps = vec![];
         let group_inputs = vec![];
         let group_outputs = vec![];
+        let data_loader = new_mock_data_loader();
         let mut load_cell = LoadCell::new(
+            &data_loader,
             &outputs,
             &resolved_inputs,
             &resolved_cell_deps,
@@ -425,7 +449,9 @@ mod tests {
         let resolved_cell_deps = vec![];
         let group_inputs = vec![];
         let group_outputs = vec![];
+        let data_loader = new_mock_data_loader();
         let mut load_cell = LoadCell::new(
+            &data_loader,
             &outputs,
             &resolved_inputs,
             &resolved_cell_deps,
@@ -489,7 +515,9 @@ mod tests {
         let resolved_cell_deps = vec![];
         let group_inputs = vec![];
         let group_outputs = vec![];
+        let data_loader = new_mock_data_loader();
         let mut load_cell = LoadCell::new(
+            &data_loader,
             &outputs,
             &resolved_inputs,
             &resolved_cell_deps,
@@ -540,7 +568,9 @@ mod tests {
         let resolved_cell_deps = vec![];
         let group_inputs = vec![];
         let group_outputs = vec![];
+        let data_loader = new_mock_data_loader();
         let mut load_cell = LoadCell::new(
+            &data_loader,
             &outputs,
             &resolved_inputs,
             &resolved_cell_deps,
@@ -560,26 +590,6 @@ mod tests {
         }
     }
 
-    struct MockDataLoader {
-        headers: HashMap<Byte32, HeaderView>,
-    }
-
-    impl CellDataProvider for MockDataLoader {
-        fn get_cell_data(&self, _out_point: &OutPoint) -> Option<Bytes> {
-            None
-        }
-
-        fn get_cell_data_hash(&self, _out_point: &OutPoint) -> Option<Byte32> {
-            None
-        }
-    }
-
-    impl HeaderProvider for MockDataLoader {
-        fn get_header(&self, block_hash: &Byte32) -> Option<HeaderView> {
-            self.headers.get(block_hash).cloned()
-        }
-    }
-
     fn _test_load_header(data: &[u8]) -> Result<(), TestCaseError> {
         let mut machine = DefaultCoreMachine::<u64, SparseMemory<u64>>::default();
         let size_addr: u64 = 0;
@@ -878,7 +888,9 @@ mod tests {
         let resolved_cell_deps = vec![];
         let group_inputs = vec![];
         let group_outputs = vec![];
+        let data_loader = new_mock_data_loader();
         let mut load_cell = LoadCell::new(
+            &data_loader,
             &outputs,
             &resolved_inputs,
             &resolved_cell_deps,
@@ -1081,8 +1093,7 @@ mod tests {
         let dep_cell_data = Bytes::from(data.to_owned());
         let dep_cell = build_cell_meta(10000, dep_cell_data);
 
-        let store = new_store();
-        let data_loader = DataLoaderWrapper::new(&store);
+        let data_loader = new_mock_data_loader();
         let outputs = vec![];
         let resolved_inputs = vec![];
         let resolved_cell_deps = vec![dep_cell];
@@ -1138,8 +1149,7 @@ mod tests {
         let dep_cell_data = Bytes::from(data.to_owned());
         let dep_cell = build_cell_meta(10000, dep_cell_data);
 
-        let store = new_store();
-        let data_loader = DataLoaderWrapper::new(&store);
+        let data_loader = new_mock_data_loader();
         let outputs = vec![];
         let resolved_inputs = vec![];
         let resolved_deps = vec![dep_cell];
@@ -1208,8 +1218,7 @@ mod tests {
         let dep_cell_data = Bytes::from(data);
         let dep_cell = build_cell_meta(10000, dep_cell_data);
 
-        let store = new_store();
-        let data_loader = DataLoaderWrapper::new(&store);
+        let data_loader = new_mock_data_loader();
         let outputs = vec![];
         let resolved_inputs = vec![];
         let resolved_cell_deps = vec![dep_cell];
@@ -1259,8 +1268,7 @@ mod tests {
         let dep_cell_data = Bytes::from(data.to_owned());
         let dep_cell = build_cell_meta(10000, dep_cell_data);
 
-        let store = new_store();
-        let data_loader = DataLoaderWrapper::new(&store);
+        let data_loader = new_mock_data_loader();
         let outputs = vec![];
         let resolved_inputs = vec![];
         let resolved_cell_deps = vec![dep_cell];
@@ -1315,8 +1323,7 @@ mod tests {
         let dep_cell_data = Bytes::from(data.to_vec());
         let dep_cell = build_cell_meta(10000, dep_cell_data);
 
-        let store = new_store();
-        let data_loader = DataLoaderWrapper::new(&store);
+        let data_loader = new_mock_data_loader();
         let outputs = vec![];
         let resolved_inputs = vec![];
         let resolved_cell_deps = vec![dep_cell];
@@ -1358,8 +1365,7 @@ mod tests {
         let dep_cell_data = Bytes::from(data.to_vec());
         let dep_cell = build_cell_meta(10000, dep_cell_data);
 
-        let store = new_store();
-        let data_loader = DataLoaderWrapper::new(&store);
+        let data_loader = new_mock_data_loader();
         let outputs = vec![];
         let resolved_inputs = vec![];
         let resolved_cell_deps = vec![dep_cell];
@@ -1404,8 +1410,7 @@ mod tests {
         let dep_cell_data = Bytes::from(data);
         let dep_cell = build_cell_meta(10000, dep_cell_data);
 
-        let store = new_store();
-        let data_loader = DataLoaderWrapper::new(&store);
+        let data_loader = new_mock_data_loader();
         let outputs = vec![];
         let resolved_inputs = vec![];
         let resolved_cell_deps = vec![dep_cell];
